import os
import asyncio
from os.path import exists

from strawberry.asgi import GraphQL
from starlette.applications import Starlette

from services.rediscache import redis
from resolvers.listener import start as listener_start, stop as listener_stop
from resolvers.schema import schema
from settings import DEV_SERVER_PID_FILE_NAME, SENTRY_DSN, MODE


async def start_up():
    if MODE == "dev":
        if exists(DEV_SERVER_PID_FILE_NAME):
            await redis.connect()
            return
        else:
            with open(DEV_SERVER_PID_FILE_NAME, "w", encoding="utf-8") as f:
                f.write(str(os.getpid()))
    else:
        await redis.connect()
        notification_service_task = asyncio.create_task(listener_start())
        print(f"[main] {notification_service_task}")

    try:
        import sentry_sdk

        sentry_sdk.init(SENTRY_DSN)
    except Exception as e:
        print("[sentry] init error")
        print(e)


async def shutdown():
    listener_stop()
    await redis.disconnect()


app = Starlette(debug=True, on_startup=[start_up], on_shutdown=[shutdown])
app.mount("/", GraphQL(schema, debug=True))
